@@ -235,12 +235,22 @@ def parse_recurrence(time_string):
 
 
 class CalendarEntryToStringWrapper(googlecl.base.BaseEntryToStringWrapper):
+  def __init__(self, entry, config):
+    """Initialize a CalendarEntry wrapper.
+
+    Args:
+      entry: CalendarEntry to interpret to strings.
+      config: Configuration parser. Needed for some values.
+    """
+    googlecl.base.BaseEntryToStringWrapper.__init__(self, entry)
+    self.config_parser = config
+
   @property
   def when(self):
     """When event takes place."""
     start_date, end_date, freq = get_datetimes(self.entry)
-    print_format = googlecl.CONFIG.lazy_get(SECTION_HEADER,
-                                            'date_print_format')
+    print_format = self.config_parser.lazy_get(SECTION_HEADER,
+                                                      'date_print_format')
     start_text = time.strftime(print_format, start_date)
     end_text = time.strftime(print_format, end_date)
     value = start_text + ' - ' + end_text
@@ -277,9 +287,9 @@ def _list(client, options, args):
 
     for entry in single_events:
       print googlecl.base.compile_entry_string(
-                                            CalendarEntryToStringWrapper(entry),
-                                            options.fields.split(','),
-                                            delimiter=options.delimiter)
+                            CalendarEntryToStringWrapper(entry, client.config),
+                            options.fields.split(','),
+                            delimiter=options.delimiter)
 
 
 #===============================================================================
@@ -356,10 +366,10 @@ def _run_delete(client, options, args):
       if date_range.specified_as_range:
         # if the user specified a date that was a range...
         client.delete_recurring_events(recurring_events, date_range.start,
-                                       date_range.end, cal.user)
+                                       date_range.end, cal.user, options.prompt)
       else:
         client.delete_recurring_events(recurring_events, date_range.start,
-                                       None, cal.user)
+                                       None, cal.user, options.prompt)
     if not (single_events or recurring_events):
       LOG.warning('No events found that match your options!')
 
