@@ -20,40 +20,40 @@ __author__ = 'tom.h.miller@gmail.com (Tom Miller)'
 import gdata.youtube
 import logging
 import os
-import googlecl
+import googlecl.base
 import googlecl.service
 from googlecl.youtube import SECTION_HEADER
 from gdata.youtube.service import YouTubeService
-
+from googlecl import safe_encode
 
 LOG = logging.getLogger(googlecl.youtube.LOGGER_NAME)
 
 
 class YouTubeServiceCL(YouTubeService, googlecl.service.BaseServiceCL):
-  
+
   """Extends gdata.youtube.service.YouTubeService for the command line.
-  
+
   This class adds some features focused on using YouTube via an installed app
   with a command line interface.
-  
+
   """
-  
-  def __init__(self):
+
+  def __init__(self, config):
     """Constructor."""
     YouTubeService.__init__(self)
-    self._set_params(SECTION_HEADER)
-  
+    googlecl.service.BaseServiceCL.__init__(self, SECTION_HEADER, config)
+
   def categorize_videos(self, video_entries, category):
     """Change the categories of a list of videos to a single category.
-    
+
     If the update fails with a request error, a message is printed to screen.
     Usually, valid category strings are the first word of the category as seen
     on YouTube (e.g. "Film" for "Film & Animation")
-    
+
     Keyword arguments:
-      video_entries: List of YouTubeVideoEntry objects. 
+      video_entries: List of YouTubeVideoEntry objects.
       category: String representation of category.
-    
+
     """
     for video in video_entries:
       video.media.category = build_category(category)
@@ -68,20 +68,21 @@ class YouTubeServiceCL(YouTubeService, googlecl.service.BaseServiceCL):
 
   CategorizeVideos = categorize_videos
 
-  def get_videos(self, user='default', title=None):
+  def get_videos(self, user='default', titles=None):
     """Get entries for videos uploaded by a user.
-    
+
     Keyword arguments:
       user: The user whose videos are being retrieved. (Default 'default')
-      title: Title that the videos should have. (Default None, for all videos)
-         
+      title: list or string Title(s) that the video(s) should have.
+             Default None, for all videos.
+
     Returns:
       List of videos that match parameters, or [] if none do.
-    
+
     """
     uri = 'http://gdata.youtube.com/feeds/api/users/' + user + '/uploads'
     return self.GetEntries(uri,
-                           title,
+                           titles,
                            converter=gdata.youtube.YouTubeVideoFeedFromString)
 
   GetVideos = get_videos
@@ -93,9 +94,9 @@ class YouTubeServiceCL(YouTubeService, googlecl.service.BaseServiceCL):
   IsTokenValid = is_token_valid
 
   def post_videos(self, paths, category, title=None, desc=None, tags=None,
-                 devtags=None):
+                 devtags=None, is_private=None):
     """Post video(s) to YouTube.
-    
+
     Keyword arguments:
       paths: List of paths to videos.
       category: YouTube category for the video.
@@ -103,62 +104,75 @@ class YouTubeServiceCL(YouTubeService, googlecl.service.BaseServiceCL):
       desc: Video summary (Default None).
       tags: Tags of the video as a string, separated by commas (Default None).
       devtags: Developer tags for the video (Default None).
-      
+
     """
-    from gdata.media import Group, Title, Description, Keywords
+    from gdata.media import Group, Title, Description, Keywords, Private
+    if isinstance(paths, basestring):
+      paths = [paths]
+    set_private = lambda private: Private() if private else None
     for path in paths:
       filename = os.path.basename(path).split('.')[0]
       my_media_group = Group(title=Title(text=title or filename),
                              description=Description(text=desc or 'A video'),
                              keywords=Keywords(text=tags),
-                             category=build_category(category))
-  
+                             category=build_category(category),
+                             private=set_private(is_private))
+
       video_entry = gdata.youtube.YouTubeVideoEntry(media=my_media_group)
       if devtags:
         taglist = devtags.replace(', ', ',')
         taglist = taglist.split(',')
         video_entry.AddDeveloperTags(taglist)
-      LOG.info('Loading ' + path)
+      LOG.info(safe_encode('Loading ' + path))
       try:
-        self.InsertVideoEntry(video_entry, path)
+        entry = self.InsertVideoEntry(video_entry, path)
       except gdata.service.RequestError, err:
-        if err.args[0]['body'].find('invalid_value') != -1:
-          err_str = 'Invalid category name'
-        else:
-          err_str = str(err)
-        LOG.error('Failed to upload video: ' + err_str)
+        LOG.error('Failed to upload video: %s' % err)
+      except gdata.youtube.service.YouTubeError, err:
+        err_str = str(err)
+        if err_str.find('path name or a file-like object') != -1:
+          err_str = safe_encode('Could not find file ' + path)
+        if (err.args[0]['body'].find('invalid_value') != -1 and
+            err.args[0]['body'].find('media:category') != -1):
+          err_str = 'Invalid category: %s' % category
+          err_str += ('\nFor a list of valid categories, see '
+                      'http://code.google.com/p/googlecl/wiki/Manual#YouTube')
+        LOG.error(err_str)
+      else:
+        LOG.info('Video uploaded: %s' % entry.GetHtmlLink().href)
 
   PostVideos = post_videos
 
   def tag_videos(self, video_entries, tags):
     """Add or remove tags on a list of videos.
-    
+
     Keyword arguments:
-      video_entries: List of YouTubeVideoEntry objects. 
-      tags: String representation of tags in a comma separated list. For how 
-            tags are generated from the string, see googlecl.service.generate_tag_sets().
-    
+      video_entries: List of YouTubeVideoEntry objects.
+      tags: String representation of tags in a comma separated list. For how
+            tags are generated from the string, see
+            googlecl.base.generate_tag_sets().
+
     """
     from gdata.media import Group, Keywords
-    remove_set, add_set, replace_tags = googlecl.service.generate_tag_sets(tags)
+    remove_set, add_set, replace_tags = googlecl.base.generate_tag_sets(tags)
     for video in video_entries:
       if not video.media:
         video.media = Group()
       if not video.media.keywords:
         video.media.keywords = Keywords()
-  
+
       # No point removing tags if the video has no keywords,
       # or we're replacing the keywords.
       if video.media.keywords.text and remove_set and not replace_tags:
         current_tags = video.media.keywords.text.replace(', ', ',')
         current_set = set(current_tags.split(','))
         video.media.keywords.text = ','.join(current_set - remove_set)
-      
+
       if replace_tags or not video.media.keywords.text:
         video.media.keywords.text = ','.join(add_set)
-      elif add_set: 
+      elif add_set:
         video.media.keywords.text += ',' + ','.join(add_set)
- 
+
       self.UpdateVideoEntry(video)
 
   TagVideos = tag_videos
@@ -169,83 +183,20 @@ SERVICE_CLASS = YouTubeServiceCL
 
 def build_category(category):
   """Build a single-item list of a YouTube category.
-  
+
   This refers to the Category of a video entry, such as "Film" or "Comedy",
   not the atom/gdata element. This does not check if the category provided
   is valid.
-  
+
   Keyword arguments:
     category: String representing the category.
-  
+
   Returns:
     A single-item list of a YouTube category (type gdata.media.Category).
-    
+
   """
   from gdata.media import Category
   return [Category(
                 text=category,
                 scheme='http://gdata.youtube.com/schemas/2007/categories.cat',
                 label=category)]
-
-
-#===============================================================================
-# Each of the following _run_* functions execute a particular task.
-#  
-# Keyword arguments:
-#  client: Client to the service being used.
-#  options: Contains all attributes required to perform the task
-#  args: Additional arguments passed in on the command line, may or may not be
-#        required
-#===============================================================================
-def _run_list(client, options, args):
-  entries = client.GetVideos(user=options.owner or options.user,
-                             title=options.title)
-  if args:
-    style_list = args[0].split(',')
-  else:
-    style_list = googlecl.get_config_option(SECTION_HEADER,
-                                            'list_style').split(',')
-  for vid in entries:
-    print googlecl.service.compile_entry_string(
-                                 googlecl.service.BaseEntryToStringWrapper(vid),
-                                 style_list,
-                                 delimiter=options.delimiter)
-
-
-def _run_post(client, options, args):
-  if not args:
-    LOG.error('Must provide path to video to post!')
-    return
-  client.PostVideos(args, title=options.title, desc=options.summary,
-                    tags=options.tags, category=options.category)
-
-
-def _run_tag(client, options, args):
-  video_entries = client.GetVideos(title=options.title)
-  if options.category:
-    client.CategorizeVideos(video_entries, options.category)
-  if options.tags:
-    client.TagVideos(video_entries, options.tags)
-
-
-def _run_delete(client, options, args):
-  entries = client.GetVideos(title=options.title)
-  client.Delete(entries, 'video',
-                googlecl.CONFIG.getboolean('GENERAL', 'delete_by_default'))
-
-
-TASKS = {'post': googlecl.service.Task('Post a video.', callback=_run_post,
-                                       required=['category', 'devkey'],
-                                       optional=['title', 'summary', 'tags'],
-                                       args_desc='PATH_TO_VIDEO'),
-         'list': googlecl.service.Task('List videos by user.',
-                                       callback=_run_list,
-                                       required='delimiter',
-                                       optional=['title', 'owner']),
-         'tag': googlecl.service.Task('Add tags to a video and/or ' +\
-                                      'change its category.',
-                                      callback=_run_tag,
-                                      required=['devkey', 'title',
-                                                ['category', 'tags']]),
-         'delete': googlecl.service.Task('Delete videos.', callback=_run_delete,
-                                         required='devkey', optional='title')}
